/*
 * Copyright 2018-2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License").
 * You may not use this file except in compliance with the License.
 * A copy of the License is located at
 *
 *     http://aws.amazon.com/apache2.0/
 *
 * or in the "license" file accompanying this file. This file is distributed
 * on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

#include <gmock/gmock.h>
#include <gtest/gtest.h>

#include <AVSCommon/AVS/Initialization/AlexaClientSDKInit.h>

#include <AACE/Test/Alexa/AlexaTestHelper.h>
#include <AACE/Engine/Alexa/AlexaEngineLogger.h>

using namespace aace::test::alexa;

class AlexaEngineLoggerTest : public ::testing::Test {
public:
    void SetUp() override
    {
        m_alexaMockFactory = AlexaTestHelper::createAlexaMockComponentFactory();
        
        // initialize the avs device SDK
        ASSERT_TRUE( alexaClientSDK::avsCommon::avs::initialization::AlexaClientSDKInit::initialize( { AlexaTestHelper::getAVSConfig() } ) ) << "Initialize AVS Device SDK Failed!";
        
        // initialized succeeded
        m_initialized = true;
    }

    void TearDown() override
    {
        if( m_initialized )
        {
            m_alexaMockFactory->shutdown();
        
            alexaClientSDK::avsCommon::avs::initialization::AlexaClientSDKInit::uninitialize();
            
            m_initialized = false;
        }
    }
    
protected:
    std::shared_ptr<AlexaMockComponentFactory> m_alexaMockFactory;

private:
    bool m_initialized = false;
};

TEST_F(AlexaEngineLoggerTest,create)
{
    auto alexaEngineLogger = aace::engine::alexa::AlexaEngineLogger::create( alexaClientSDK::avsCommon::utils::logger::Level::DEBUG0 );
    EXPECT_NE(alexaEngineLogger,nullptr) << "AlexaEngineLogger pointer is null!";
    
    alexaEngineLogger->shutdown();
}
